@@ -53,7 +53,7 @@
 extern int structured_debug;
 #endif
 /* "%code requires" blocks.  */
-#line 178 "../../jitter/example-vms/structured/structured.y"
+#line 277 "../../jitter/example-vms/structured/structured.y"
 
 /* Simplified error-reporting facilities calling structured_error, suitable to be
    called from the scanner and the parser without the complicated and
@@ -78,44 +78,47 @@ structured_parse_file (const char *input_file_name);
     STRUCTURED_EOF = 0,            /* "end of file"  */
     STRUCTURED_error = 256,        /* error  */
     STRUCTURED_UNDEF = 257,        /* "invalid token"  */
-    BEGIN_ = 258,                  /* BEGIN_  */
-    END = 259,                     /* END  */
-    SKIP = 260,                    /* SKIP  */
-    VAR = 261,                     /* VAR  */
-    PRINT = 262,                   /* PRINT  */
-    INPUT = 263,                   /* INPUT  */
-    SET_TO = 264,                  /* SET_TO  */
-    SEMICOLON = 265,               /* SEMICOLON  */
-    COMMA = 266,                   /* COMMA  */
-    IF = 267,                      /* IF  */
-    THEN = 268,                    /* THEN  */
-    ELSE = 269,                    /* ELSE  */
-    ELIF = 270,                    /* ELIF  */
-    WHILE = 271,                   /* WHILE  */
-    DO = 272,                      /* DO  */
-    REPEAT = 273,                  /* REPEAT  */
-    UNTIL = 274,                   /* UNTIL  */
-    OPEN_PAREN = 275,              /* OPEN_PAREN  */
-    CLOSE_PAREN = 276,             /* CLOSE_PAREN  */
-    VARIABLE = 277,                /* VARIABLE  */
-    DECIMAL_LITERAL = 278,         /* DECIMAL_LITERAL  */
-    TRUE = 279,                    /* TRUE  */
-    FALSE = 280,                   /* FALSE  */
-    PLUS = 281,                    /* PLUS  */
-    MINUS = 282,                   /* MINUS  */
-    TIMES = 283,                   /* TIMES  */
-    DIVIDED = 284,                 /* DIVIDED  */
-    REMAINDER = 285,               /* REMAINDER  */
-    EQUAL = 286,                   /* EQUAL  */
-    DIFFERENT = 287,               /* DIFFERENT  */
-    LESS = 288,                    /* LESS  */
-    LESS_OR_EQUAL = 289,           /* LESS_OR_EQUAL  */
-    GREATER = 290,                 /* GREATER  */
-    GREATER_OR_EQUAL = 291,        /* GREATER_OR_EQUAL  */
-    LOGICAL_OR = 292,              /* LOGICAL_OR  */
-    LOGICAL_AND = 293,             /* LOGICAL_AND  */
-    LOGICAL_NOT = 294,             /* LOGICAL_NOT  */
-    UNARY_MINUS = 295              /* UNARY_MINUS  */
+    PROCEDURE = 258,               /* PROCEDURE  */
+    RETURN = 259,                  /* RETURN  */
+    BEGIN_ = 260,                  /* BEGIN_  */
+    END = 261,                     /* END  */
+    SKIP = 262,                    /* SKIP  */
+    VAR = 263,                     /* VAR  */
+    PRINT = 264,                   /* PRINT  */
+    INPUT = 265,                   /* INPUT  */
+    SET_TO = 266,                  /* SET_TO  */
+    SEMICOLON = 267,               /* SEMICOLON  */
+    COMMA = 268,                   /* COMMA  */
+    IF = 269,                      /* IF  */
+    THEN = 270,                    /* THEN  */
+    ELSE = 271,                    /* ELSE  */
+    ELIF = 272,                    /* ELIF  */
+    WHILE = 273,                   /* WHILE  */
+    DO = 274,                      /* DO  */
+    REPEAT = 275,                  /* REPEAT  */
+    UNTIL = 276,                   /* UNTIL  */
+    OPEN_PAREN = 277,              /* OPEN_PAREN  */
+    CLOSE_PAREN = 278,             /* CLOSE_PAREN  */
+    UNDEFINED = 279,               /* UNDEFINED  */
+    VARIABLE = 280,                /* VARIABLE  */
+    DECIMAL_LITERAL = 281,         /* DECIMAL_LITERAL  */
+    TRUE = 282,                    /* TRUE  */
+    FALSE = 283,                   /* FALSE  */
+    PLUS = 284,                    /* PLUS  */
+    MINUS = 285,                   /* MINUS  */
+    TIMES = 286,                   /* TIMES  */
+    DIVIDED = 287,                 /* DIVIDED  */
+    REMAINDER = 288,               /* REMAINDER  */
+    EQUAL = 289,                   /* EQUAL  */
+    DIFFERENT = 290,               /* DIFFERENT  */
+    LESS = 291,                    /* LESS  */
+    LESS_OR_EQUAL = 292,           /* LESS_OR_EQUAL  */
+    GREATER = 293,                 /* GREATER  */
+    GREATER_OR_EQUAL = 294,        /* GREATER_OR_EQUAL  */
+    LOGICAL_OR = 295,              /* LOGICAL_OR  */
+    LOGICAL_AND = 296,             /* LOGICAL_AND  */
+    LOGICAL_NOT = 297,             /* LOGICAL_NOT  */
+    UNARY_MINUS = 298              /* UNARY_MINUS  */
   };
   typedef enum structured_tokentype structured_token_kind_t;
 #endif
@@ -123,57 +126,61 @@ structured_parse_file (const char *input_file_name);
 #define STRUCTURED_EOF 0
 #define STRUCTURED_error 256
 #define STRUCTURED_UNDEF 257
-#define BEGIN_ 258
-#define END 259
-#define SKIP 260
-#define VAR 261
-#define PRINT 262
-#define INPUT 263
-#define SET_TO 264
-#define SEMICOLON 265
-#define COMMA 266
-#define IF 267
-#define THEN 268
-#define ELSE 269
-#define ELIF 270
-#define WHILE 271
-#define DO 272
-#define REPEAT 273
-#define UNTIL 274
-#define OPEN_PAREN 275
-#define CLOSE_PAREN 276
-#define VARIABLE 277
-#define DECIMAL_LITERAL 278
-#define TRUE 279
-#define FALSE 280
-#define PLUS 281
-#define MINUS 282
-#define TIMES 283
-#define DIVIDED 284
-#define REMAINDER 285
-#define EQUAL 286
-#define DIFFERENT 287
-#define LESS 288
-#define LESS_OR_EQUAL 289
-#define GREATER 290
-#define GREATER_OR_EQUAL 291
-#define LOGICAL_OR 292
-#define LOGICAL_AND 293
-#define LOGICAL_NOT 294
-#define UNARY_MINUS 295
+#define PROCEDURE 258
+#define RETURN 259
+#define BEGIN_ 260
+#define END 261
+#define SKIP 262
+#define VAR 263
+#define PRINT 264
+#define INPUT 265
+#define SET_TO 266
+#define SEMICOLON 267
+#define COMMA 268
+#define IF 269
+#define THEN 270
+#define ELSE 271
+#define ELIF 272
+#define WHILE 273
+#define DO 274
+#define REPEAT 275
+#define UNTIL 276
+#define OPEN_PAREN 277
+#define CLOSE_PAREN 278
+#define UNDEFINED 279
+#define VARIABLE 280
+#define DECIMAL_LITERAL 281
+#define TRUE 282
+#define FALSE 283
+#define PLUS 284
+#define MINUS 285
+#define TIMES 286
+#define DIVIDED 287
+#define REMAINDER 288
+#define EQUAL 289
+#define DIFFERENT 290
+#define LESS 291
+#define LESS_OR_EQUAL 292
+#define GREATER 293
+#define GREATER_OR_EQUAL 294
+#define LOGICAL_OR 295
+#define LOGICAL_AND 296
+#define LOGICAL_NOT 297
+#define UNARY_MINUS 298
 
 /* Value type.  */
 #if ! defined STRUCTURED_STYPE && ! defined STRUCTURED_STYPE_IS_DECLARED
 union STRUCTURED_STYPE
 {
-#line 193 "../../jitter/example-vms/structured/structured.y"
+#line 292 "../../jitter/example-vms/structured/structured.y"
 
   jitter_int literal;
   structured_variable variable;
   struct structured_expression *expression;
   struct structured_statement *statement;
+  struct structured_sequence *pointers;
 
-#line 177 "../../jitter/example-vms/structured/structured-parser.h"
+#line 184 "../../jitter/example-vms/structured/structured-parser.h"
 
 };
 typedef union STRUCTURED_STYPE STRUCTURED_STYPE;
