@@ -716,16 +716,22 @@ pkl_ast_node pkl_ast_make_trimmer (pkl_ast ast,
    BASE must point to a PKL_AST_ARRAY node.
 
    INDEX must point to an expression whose evaluation is the offset of
-   the element into the field, in units of the field's SIZE.  */
+   the element into the field, in units of the field's SIZE.
+
+   IS_INDEXED is a boolean indicating whether the indexer is
+   immediately indexed with another [] operator.  This is used in the
+   `gen' pass for certain optimizations.  */
 
 #define PKL_AST_INDEXER_ENTITY(AST) ((AST)->indexer.entity)
 #define PKL_AST_INDEXER_INDEX(AST) ((AST)->indexer.index)
+#define PKL_AST_INDEXER_IS_INDEXED(AST) ((AST)->indexer.is_indexed)
 
 struct pkl_ast_indexer
 {
   struct pkl_ast_common common;
   union pkl_ast_node *entity;
   union pkl_ast_node *index;
+  int is_indexed;
 };
 
 pkl_ast_node pkl_ast_make_indexer (pkl_ast ast,
@@ -1426,6 +1432,10 @@ pkl_ast_node pkl_ast_make_format_arg (pkl_ast ast, pkl_ast_node exp);
    variable.  This is used to disable the de-proceduring of function
    values.
 
+   IS_INDEXED is a boolean indicating whether the variable is
+   immediately indexed with a [] operator.  This is used in the `gen'
+   pass for certain optimizations.
+
    FUNCTION is the function immediately enclosing the variable
    reference, or NULL.
 
@@ -1441,6 +1451,7 @@ pkl_ast_node pkl_ast_make_format_arg (pkl_ast ast, pkl_ast_node exp);
 #define PKL_AST_VAR_FUNCTION(AST) ((AST)->var.function)
 #define PKL_AST_VAR_FUNCTION_BACK(AST) ((AST)->var.function_back)
 #define PKL_AST_VAR_IS_PARENTHESIZED(AST) ((AST)->var.is_parenthesized)
+#define PKL_AST_VAR_IS_INDEXED(AST) ((AST)->var.is_indexed)
 
 struct pkl_ast_var
 {
@@ -1452,6 +1463,7 @@ struct pkl_ast_var
   int over;
   int is_recursive;
   int is_parenthesized;
+  int is_indexed;
 
   union pkl_ast_node *function;
   int function_back;
@@ -2025,4 +2037,6 @@ pkl_ast_node pkl_ast_reverse (pkl_ast_node ast);
 
 void pkl_ast_print (FILE *fp, pkl_ast_node ast);
 
+char *pkl_ast_format_loc (pkl_ast ast, pkl_ast_loc loc);
+
 #endif /* ! PKL_AST_H */
