@@ -1,6 +1,6 @@
 /* pvm-val.c - Values for the PVM.  */
 
-/* Copyright (C) 2019, 2020, 2021 Jose E. Marchesi */
+/* Copyright (C) 2019, 2020, 2021, 2022 Jose E. Marchesi */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -40,6 +40,22 @@ static pvm_val string_type;
 static pvm_val void_type;
 static pvm_val any_type;
 
+/* We are currently only supporting a relatively small number of
+   integral types, i.e. signed and unsigned types of sizes 1 to 64
+   bits.  It is therefore possible to cache these types to avoid
+   allocating them again and again.
+
+   The contents of this table are initialized in pvm_val_initialize
+   and finalized in pvm_val_finalize.  New types are installed by
+   pvm_make_integral_type.
+
+   Note that the first entry of the table is unused; it would
+   correspond to an integer of "zero" bits.  This is more efficient
+   than correcting the index at every access at the cost of only
+   64-bits.  */
+
+static pvm_val common_int_types[65][2];
+
 pvm_val
 pvm_make_int (int32_t value, int size)
 {
@@ -68,6 +84,31 @@ pvm_make_ulong (uint64_t value, int size)
   return PVM_MAKE_LONG_ULONG (value, size, PVM_VAL_TAG_ULONG);
 }
 
+
+pvm_val
+pvm_make_signed_integral (int64_t value, int size)
+{
+  if (size > 64)
+    return PK_NULL;
+
+  if (size <= 32)
+    return PVM_MAKE_INT ((int32_t) value, size);
+  else
+    return PVM_MAKE_LONG (value, size);
+}
+
+pvm_val
+pvm_make_unsigned_integral (uint64_t value, int size)
+{
+  if (size > 64)
+    return PK_NULL;
+
+  if (size <= 32)
+    return PVM_MAKE_UINT ((uint32_t) value, size);
+  else
+    return PVM_MAKE_ULONG (value, size);
+}
+
 static pvm_val_box
 pvm_make_box (uint8_t tag)
 {
@@ -197,26 +238,28 @@ pvm_array_set (pvm_val arr, pvm_val idx, pvm_val val)
 {
   size_t index = PVM_VAL_ULONG (idx);
   size_t nelem = PVM_VAL_ULONG (PVM_VAL_ARR_NELEM (arr));
-  size_t elem_boffset;
   size_t i;
+  ssize_t size_diff;
 
   /* Make sure that the given index is within bounds.  */
   if (index >= nelem)
     return 0;
 
+  /* Calculate the difference of size introduced by the new
+     elemeent.  */
+  size_diff = ((ssize_t) pvm_sizeof (val)
+               - (ssize_t) pvm_sizeof (PVM_VAL_ARR_ELEM_VALUE (arr, index)));
+
   /* Update the element with the given value.  */
   PVM_VAL_ARR_ELEM_VALUE (arr, index) = val;
 
   /* Recalculate the bit-offset of all the elemens following the
      element just updated.  */
-  elem_boffset
-    = (PVM_VAL_ULONG (PVM_VAL_ARR_ELEM_OFFSET (arr, index))
-       + pvm_sizeof (PVM_VAL_ARR_ELEM_VALUE (arr, index)));
-
   for (i = index + 1; i < nelem; ++i)
     {
+      size_t elem_boffset
+        = (ssize_t) PVM_VAL_ULONG (PVM_VAL_ARR_ELEM_OFFSET (arr, i)) + size_diff;
       PVM_VAL_ARR_ELEM_OFFSET (arr, i) = pvm_make_ulong (elem_boffset, 64);
-      elem_boffset += pvm_sizeof (PVM_VAL_ARR_ELEM_VALUE (arr, i));
     }
 
   return 1;
@@ -414,8 +457,8 @@ pvm_make_type (enum pvm_type_code code)
   return PVM_BOX (box);
 }
 
-pvm_val
-pvm_make_integral_type (pvm_val size, pvm_val signed_p)
+static inline pvm_val
+pvm_make_integral_type_1 (pvm_val size, pvm_val signed_p)
 {
   pvm_val itype = pvm_make_type (PVM_TYPE_INTEGRAL);
 
@@ -424,6 +467,22 @@ pvm_make_integral_type (pvm_val size, pvm_val signed_p)
   return itype;
 }
 
+pvm_val
+pvm_make_integral_type (pvm_val size, pvm_val signed_p)
+{
+  uint64_t bits = PVM_VAL_ULONG (size);
+  int32_t sign = PVM_VAL_INT (signed_p);
+  pvm_val itype = common_int_types[bits][sign];
+
+  if (itype == PVM_NULL)
+    {
+      itype = pvm_make_integral_type_1 (size, signed_p);
+      common_int_types[bits][sign] = itype;
+    }
+
+  return itype;
+}
+
 pvm_val
 pvm_make_string_type (void)
 {
@@ -973,11 +1032,12 @@ print_unit_name (uint64_t unit)
 }
 
 #define PVM_PRINT_VAL_1(...)                    \
-  pvm_print_val_1 (vm, depth, mode, base, indent, acutoff, flags, __VA_ARGS__)
+  pvm_print_val_1 (vm, depth, mode, base, indent, acutoff, flags, \
+                   exit_exception, __VA_ARGS__)
 
 static void
 pvm_print_val_1 (pvm vm, int depth, int mode, int base, int indent,
-                 int acutoff, uint32_t flags,
+                 int acutoff, uint32_t flags, pvm_val *exit_exception,
                  pvm_val val, int ndepth)
 {
   const char *long64_fmt, *long_fmt;
@@ -1027,9 +1087,9 @@ pvm_print_val_1 (pvm vm, int depth, int mode, int base, int indent,
       break;
     case 16:
       long64_fmt = "0x%" PRIx64 "L";
-      long_fmt = "(int<%d>) %" PRIx64;
+      long_fmt = "(int<%d>) 0x%" PRIx64;
       ulong64_fmt = "0x%" PRIx64 "UL";
-      ulong_fmt = "(uint<%d>) %" PRIx64;
+      ulong_fmt = "(uint<%d>) 0x%" PRIx64;
       int32_fmt = "0x%" PRIx32;
       int16_fmt = "0x%" PRIx32 "H";
       int8_fmt = "0x%" PRIx32 "B";
@@ -1083,7 +1143,8 @@ pvm_print_val_1 (pvm vm, int depth, int mode, int base, int indent,
       if (base == 2)
         {
           pk_puts ("0b");
-          pk_print_binary (pk_puts, ulongval, size, 1);
+          pk_print_binary (pk_puts, ulongval, size, /*signed_p*/ 1,
+                           /*use_suffix_p*/ 1);
         }
       else
         {
@@ -1112,7 +1173,8 @@ pvm_print_val_1 (pvm vm, int depth, int mode, int base, int indent,
       if (base == 2)
         {
           pk_puts ("0b");
-          pk_print_binary (pk_puts, (uint64_t) uintval, size, 1);
+          pk_print_binary (pk_puts, (uint64_t) uintval, size, /*signed*/ 1,
+                           /*use_suffix_p*/ 1);
         }
       else
         {
@@ -1141,7 +1203,8 @@ pvm_print_val_1 (pvm vm, int depth, int mode, int base, int indent,
       if (base == 2)
         {
           pk_puts ("0b");
-          pk_print_binary (pk_puts, ulongval, size, 0);
+          pk_print_binary (pk_puts, ulongval, size, /*signed_p*/ 0,
+                           /*use_suffix_p*/ 1);
         }
       else
         {
@@ -1163,7 +1226,8 @@ pvm_print_val_1 (pvm vm, int depth, int mode, int base, int indent,
       if (base == 2)
         {
           pk_puts ("0b");
-          pk_print_binary (pk_puts, uintval, size, 0);
+          pk_print_binary (pk_puts, uintval, size, /*signed_p*/ 0,
+                           /*use_suffix_p*/ 1);
         }
       else
         {
@@ -1307,7 +1371,7 @@ pvm_print_val_1 (pvm vm, int depth, int mode, int base, int indent,
          then use it, unless the PVM is configured to not do so.  */
       if (pprint)
         {
-          if (pvm_call_pretty_printer (vm, val))
+          if (pvm_call_pretty_printer (vm, val, exit_exception))
             return;
         }
 
@@ -1509,14 +1573,17 @@ pvm_print_val_1 (pvm vm, int depth, int mode, int base, int indent,
 #undef PVM_PRINT_VAL_1
 
 void
-pvm_print_val (pvm vm, pvm_val val)
+pvm_print_val (pvm vm, pvm_val val, pvm_val *exit_exception)
 {
+  if (exit_exception)
+    *exit_exception = PVM_NULL;
   pvm_print_val_1 (vm,
                    pvm_odepth (vm), pvm_omode (vm),
                    pvm_obase (vm), pvm_oindent (vm),
                    pvm_oacutoff (vm),
                    (pvm_omaps (vm) << (PVM_PRINT_F_MAPS - 1)
                     | (pvm_pretty_print (vm) << (PVM_PRINT_F_PPRINT - 1))),
+                   exit_exception,
                    val,
                    0 /* ndepth */);
 }
@@ -1525,11 +1592,15 @@ void
 pvm_print_val_with_params (pvm vm, pvm_val val,
                            int depth,int mode, int base,
                            int indent, int acutoff,
-                           uint32_t flags)
+                           uint32_t flags, pvm_val *exit_exception)
 {
+  if (exit_exception)
+    *exit_exception = PVM_NULL;
+
   pvm_print_val_1 (vm,
                    depth, mode, base, indent, acutoff,
                    flags,
+                   exit_exception,
                    val,
                    0 /* ndepth */);
 }
@@ -1560,6 +1631,8 @@ pvm_typeof (pvm_val val)
     type = PVM_VAL_ARR_TYPE (val);
   else if (PVM_IS_SCT (val))
     type = PVM_VAL_SCT_TYPE (val);
+  else if (PVM_IS_TYP (val))
+    type = val;
   else
     assert (0);
 
@@ -1638,14 +1711,14 @@ pvm_print_string (pvm_val string)
    corresponding to the struct VAL.  */
 
 int
-pvm_call_pretty_printer (pvm vm, pvm_val val)
+pvm_call_pretty_printer (pvm vm, pvm_val val, pvm_val *exit_exception)
 {
   pvm_val cls = pvm_get_struct_method (val, "_print");
 
   if (cls == PVM_NULL)
     return 0;
 
-  pvm_call_closure (vm, cls, val, PVM_NULL);
+  pvm_call_closure (vm, cls, exit_exception, val, PVM_NULL);
   return 1;
 }
 
@@ -1653,25 +1726,39 @@ pvm_call_pretty_printer (pvm vm, pvm_val val)
    definition of the struct Exception in pkl-rt.pk.  */
 
 pvm_val
-pvm_make_exception (int code, char *message, int exit_status)
+pvm_make_exception (int code, const char *name, int exit_status,
+                    const char *location, const char *msg)
 {
-  pvm_val nfields = pvm_make_ulong (3, 64);
+  pvm_val nfields = pvm_make_ulong (5, 64);
   pvm_val nmethods = pvm_make_ulong (0, 64);
   pvm_val struct_name = pvm_make_string ("Exception");
   pvm_val code_name = pvm_make_string ("code");
-  pvm_val msg_name = pvm_make_string ("msg");
+  pvm_val name_name = pvm_make_string ("name");
   pvm_val exit_status_name = pvm_make_string ("exit_status");
+  pvm_val location_name = pvm_make_string ("location");
+  pvm_val msg_name = pvm_make_string ("msg");
   pvm_val *field_names, *field_types, type;
   pvm_val exception;
 
   pvm_allocate_struct_attrs (nfields, &field_names, &field_types);
 
   field_names[0] = code_name;
-  field_types[0] = pvm_make_integral_type (32, 1);
+  field_types[0] = pvm_make_integral_type (pvm_make_ulong (32, 64),
+                                           PVM_MAKE_INT (1, 32));
 
-  field_names[1] = msg_name;
+  field_names[1] = name_name;
   field_types[1] = pvm_make_string_type ();
 
+  field_names[2] = exit_status_name;
+  field_types[2] = pvm_make_integral_type (pvm_make_ulong (32, 64),
+                                           PVM_MAKE_INT (1, 32));
+
+  field_names[3] = location_name;
+  field_types[3] = pvm_make_string_type ();
+
+  field_names[4] = msg_name;
+  field_types[4] = pvm_make_string_type ();
+
   type = pvm_make_struct_type (nfields, struct_name,
                                field_names, field_types);
 
@@ -1681,14 +1768,22 @@ pvm_make_exception (int code, char *message, int exit_status)
   PVM_VAL_SCT_FIELD_VALUE (exception, 0)
     = PVM_MAKE_INT (code, 32);
 
-  PVM_VAL_SCT_FIELD_NAME (exception, 1) = msg_name;
+  PVM_VAL_SCT_FIELD_NAME (exception, 1) = name_name;
   PVM_VAL_SCT_FIELD_VALUE (exception, 1)
-    = pvm_make_string (message);
+    = pvm_make_string (name);
 
   PVM_VAL_SCT_FIELD_NAME (exception, 2) = exit_status_name;
   PVM_VAL_SCT_FIELD_VALUE (exception, 2)
     = PVM_MAKE_INT (exit_status, 32);
 
+  PVM_VAL_SCT_FIELD_NAME (exception, 3) = location_name;
+  PVM_VAL_SCT_FIELD_VALUE (exception, 3)
+    = pvm_make_string (location == NULL ? "" : location);
+
+  PVM_VAL_SCT_FIELD_NAME (exception, 4) = msg_name;
+  PVM_VAL_SCT_FIELD_VALUE (exception, 4)
+    = pvm_make_string (msg == NULL ? "" : msg);
+
   return exception;
 }
 
@@ -1701,13 +1796,21 @@ pvm_val_cls_program (pvm_val cls)
 void
 pvm_val_initialize (void)
 {
+  uint64_t i;
+  int32_t j;
+
   pvm_alloc_add_gc_roots (&string_type, 1);
   pvm_alloc_add_gc_roots (&void_type, 1);
   pvm_alloc_add_gc_roots (&any_type, 1);
+  pvm_alloc_add_gc_roots (&common_int_types, 65 * 2);
 
   string_type = pvm_make_type (PVM_TYPE_STRING);
   void_type = pvm_make_type (PVM_TYPE_VOID);
   any_type = pvm_make_type (PVM_TYPE_ANY);
+
+  for (i = 0; i < 65; ++i)
+    for (j = 0; j < 2; ++j)
+      common_int_types[i][j] = PVM_NULL;
 }
 
 void
@@ -1716,4 +1819,5 @@ pvm_val_finalize (void)
   pvm_alloc_remove_gc_roots (&string_type, 1);
   pvm_alloc_remove_gc_roots (&void_type, 1);
   pvm_alloc_remove_gc_roots (&any_type, 1);
+  pvm_alloc_remove_gc_roots (&common_int_types, 65 * 2);
 }
